@@ -20,10 +20,15 @@
 */
 
 #include <config.h>
+#include "gtkhtml.h"
 #include "htmlcluealigned.h"
 #include "htmlcluev.h"
 #include "htmlengine.h"
 #include "htmlpainter.h"
+#include "htmlcolor.h"
+#include "htmlcolorset.h"
+#include "htmlsettings.h"
+#include "htmlstyle.h"
 
 
 HTMLClueVClass html_cluev_class;
@@ -90,7 +95,7 @@ cluev_next_aligned (HTMLObject *aclue)
 static gint
 get_lmargin (HTMLObject *o, HTMLPainter *painter)
 {
-	return HTML_CLUEV (o)->padding * html_painter_get_pixel_size (painter)
+	return (HTML_CLUEV (o)->padding + HTML_CLUEV (o)->border_width) * html_painter_get_pixel_size (painter)
 		+ (o->parent ?  html_object_get_left_margin (o->parent, painter, o->y, TRUE) : 0);
 }
 
@@ -136,7 +141,7 @@ html_cluev_do_layout (HTMLObject *o, HTMLPainter *painter, gboolean calc_size, G
 	clue = HTML_CLUE (o);
 
 	pixel_size = html_painter_get_pixel_size (painter);
-	padding    = pixel_size * cluev->padding;
+	padding    = pixel_size * (cluev->padding + cluev->border_width);
 	padding2   = 2 * padding;
 
 	old_width = o->width;
@@ -192,6 +197,9 @@ html_cluev_do_layout (HTMLObject *o, HTMLPainter *painter, gboolean calc_size, G
 			break;
 		case HTML_CLEAR_NONE:
 			break;
+		case HTML_CLEAR_INHERIT:
+			/* TODO */
+			break;
 		}
 
 		o->ascent = clue->curr->y;
@@ -288,8 +296,19 @@ copy (HTMLObject *self,
 
 	HTML_CLUEV (dest)->padding = HTML_CLUEV (self)->padding;
 
+	HTML_CLUEV (dest)->border_width = HTML_CLUEV (self)->border_width;
+	HTML_CLUEV (dest)->border_style = HTML_CLUEV (self)->border_style;
+	HTML_CLUEV (dest)->border_color = HTML_CLUEV (self)->border_color;
+	if (HTML_CLUEV (dest)->border_color)
+		html_color_ref (HTML_CLUEV (dest)->border_color);
+	HTML_CLUEV (dest)->background_color = HTML_CLUEV (self)->background_color;
+	if (HTML_CLUEV (dest)->background_color)
+		html_color_ref (HTML_CLUEV (dest)->background_color);
+
 	HTML_CLUEV (dest)->align_left_list = NULL;
 	HTML_CLUEV (dest)->align_right_list = NULL;
+
+	HTML_CLUEV (dest)->dir = HTML_CLUEV (self)->dir;
 }
 
 static gboolean
@@ -303,7 +322,7 @@ calc_min_width (HTMLObject *o,
 		HTMLPainter *painter)
 {
 	return (* HTML_OBJECT_CLASS (parent_class)->calc_min_width) (o, painter)
-		+ 2 * html_painter_get_pixel_size (painter) * HTML_CLUEV (o)->padding;
+		+ 2 * html_painter_get_pixel_size (painter) * (HTML_CLUEV (o)->padding + HTML_CLUEV (o)->border_width);
 }
 
 static gint
@@ -311,7 +330,7 @@ calc_preferred_width (HTMLObject *o,
 		      HTMLPainter *painter)
 {
 	return (* HTML_OBJECT_CLASS (parent_class)->calc_preferred_width) (o, painter)
-		+ 2 * html_painter_get_pixel_size (painter) * HTML_CLUEV (o)->padding;
+		+ 2 * html_painter_get_pixel_size (painter) * (HTML_CLUEV (o)->padding + HTML_CLUEV (o)->border_width);
 }
 
 static void
@@ -320,7 +339,7 @@ set_max_width (HTMLObject *o, HTMLPainter *painter, gint max_width)
 	HTMLObject *obj;
 
 	o->max_width = max_width;
-	max_width   -= 2 * HTML_CLUEV (o)->padding * html_painter_get_pixel_size (painter);
+	max_width   -= 2 * (HTML_CLUEV (o)->padding + HTML_CLUEV (o)->border_width) * html_painter_get_pixel_size (painter);
 	for (obj = HTML_CLUE (o)->head; obj != NULL; obj = obj->next)
 		html_object_set_max_width (obj, painter, max_width);
 }
@@ -329,7 +348,6 @@ static void
 set_max_height (HTMLObject *o, HTMLPainter *painter, gint height)
 {
 	HTMLClue *clue = HTML_CLUE (o);
-	HTMLObject *obj;
 
 	html_object_calc_size (o, painter, NULL);
 	if (o->ascent < height) {
@@ -359,6 +377,20 @@ draw (HTMLObject *o,
       gint tx, gint ty)
 {
 	HTMLObject *aclue;
+	HTMLClueV *cluev;
+	GdkRectangle paint;
+
+	cluev = HTML_CLUEV (o);
+	
+	if (!html_object_intersect (o, &paint, x, y, width, height))
+		return;
+
+	if (cluev->background_color) {
+		html_painter_alloc_color (p, &cluev->background_color->color);
+		html_painter_draw_background (p, 
+					      &cluev->background_color->color,
+					      NULL, tx + paint.x, ty + paint.y, paint.width, paint.height, 0, 0);
+	}
 
 	HTML_OBJECT_CLASS (&html_clue_class)->draw (o,
 						    p,
@@ -366,9 +398,6 @@ draw (HTMLObject *o,
 						    width, height,
 						    tx, ty);
 
-	if (y + height < o->y - o->ascent || y > o->y + o->descent)
-		return;
-
 	tx += o->x;
 	ty += o->y - o->ascent;
 
@@ -395,6 +424,23 @@ draw (HTMLObject *o,
 				  tx + aclue->parent->x,
 				  ty + aclue->parent->y - aclue->parent->ascent);
 	}
+
+	if (cluev->border_style != HTML_BORDER_NONE && cluev->border_width > 0) {
+		GdkColor *color;
+
+		if (cluev->border_color) {
+			html_painter_alloc_color (p, &cluev->border_color->color);
+			color = &cluev->border_color->color;
+		} else
+			color = &html_colorset_get_color_allocated (GTK_HTML (p->widget)->engine->settings->color_set,
+								    p, HTMLTextColor)->color;
+		html_painter_draw_border (p, color,
+					  tx, ty, 
+					  o->width,
+					  o->ascent + o->descent,
+					  cluev->border_style,
+					  html_painter_get_pixel_size (p) * cluev->border_width);
+	}
 }
 
 static HTMLObject *
@@ -407,6 +453,7 @@ check_point (HTMLObject *self,
 	HTMLObject *p;
 	HTMLObject *obj;
 	HTMLClueAligned *clue;
+	int padding = HTML_CLUEV (self)->padding;
 
 	if (x < self->x || x >= self->x + self->width
 	    || y < self->y - self->ascent || y >= self->y + self->descent)
@@ -415,6 +462,19 @@ check_point (HTMLObject *self,
 	x = x - self->x;
 	y = y - self->y + self->ascent;
 
+	if (!for_cursor) {
+		if (x < padding || y < padding) {
+			if (offset_return)
+				*offset_return = 0;
+			return self;
+		}
+		if (x >= self->width - padding || y >= self->ascent + self->descent - padding) {
+			if (offset_return)
+				*offset_return = 1;
+			return self;
+		}
+	}
+
 	for (clue = HTML_CLUEALIGNED (HTML_CLUEV (self)->align_left_list);
 	     clue != NULL;
 	     clue = clue->next_aligned) {
@@ -479,6 +539,18 @@ check_point (HTMLObject *self,
 			return obj;
 	}
 
+	if (!for_cursor) {
+		if (x >= 0 && y >= 0 && x < self->width && y < self->ascent + self->descent) {
+			if (offset_return) {
+				if (x < self->width/2)
+					*offset_return = 0;
+				else
+					*offset_return = 1;
+			}
+			return self;
+		}
+	}
+
 	return NULL;
 }
 
@@ -581,7 +653,7 @@ get_right_margin (HTMLObject *self, HTMLPainter *painter, gint y, gboolean with_
 	gint margin;
 	
 	cluev = HTML_CLUEV (self);
-	margin = self->max_width - 2 * cluev->padding * html_painter_get_pixel_size (painter);
+	margin = self->max_width - 2 * (cluev->padding + cluev->border_width)* html_painter_get_pixel_size (painter);
 
 	if (with_aligned)
 		for (aclue = cluev->align_right_list;
@@ -613,7 +685,7 @@ find_free_area (HTMLClue *clue, HTMLPainter *painter, gint y, gint width, gint h
 	next_y = 0;
 	while (1) {
 		lmargin = indent;
-		rmargin = HTML_OBJECT (clue)->max_width - 2 * cluev->padding * html_painter_get_pixel_size (painter);
+		rmargin = HTML_OBJECT (clue)->max_width - 2 * (cluev->padding + cluev->border_width) * html_painter_get_pixel_size (painter);
 		
 		for (aclue = cluev->align_left_list; aclue != 0; aclue = cluev_next_aligned (aclue)) {
 			base_y = (aclue->y + aclue->parent->y
@@ -807,6 +879,16 @@ get_right_clear (HTMLClue *self,
 	return y;
 }
 
+static HTMLDirection
+html_cluev_real_get_direction (HTMLObject *o)
+{
+	if (HTML_CLUEV (o)->dir == HTML_DIRECTION_DERIVED && o->parent) {
+		return html_object_get_direction (o->parent);
+	}
+
+	return HTML_CLUEV (o)->dir;
+}
+
 
 void
 html_cluev_type_init (void)
@@ -839,6 +921,7 @@ html_cluev_class_init (HTMLClueVClass *klass,
 	object_class->check_point = check_point;
 	object_class->get_left_margin = get_left_margin;
 	object_class->get_right_margin = get_right_margin;
+	object_class->get_direction = html_cluev_real_get_direction;
 
 	clue_class->get_left_clear = get_left_clear;
 	clue_class->get_right_clear = get_right_clear;
@@ -869,12 +952,17 @@ html_cluev_init (HTMLClueV *cluev,
 	object->percent = percent;
 
 	clue->valign = HTML_VALIGN_BOTTOM;
-	clue->halign = HTML_HALIGN_LEFT;
+	clue->halign = HTML_HALIGN_NONE;
+	cluev->dir = HTML_DIRECTION_DERIVED;
 	clue->head = clue->tail = clue->curr = 0;
 
 	cluev->align_left_list = 0;
 	cluev->align_right_list = 0;
 	cluev->padding = 0;
+	cluev->border_style = HTML_BORDER_NONE;
+	cluev->border_width = 0;
+	cluev->border_color = NULL;
+	cluev->background_color = NULL;
 }
 
 HTMLObject *
@@ -887,3 +975,35 @@ html_cluev_new (gint x, gint y, gint percent)
 	
 	return HTML_OBJECT (cluev);
 }
+
+void
+html_cluev_set_style (HTMLClueV *cluev, HTMLStyle *style)
+{
+	if (style != NULL) {
+		if (cluev->border_color)
+			html_color_unref (cluev->border_color);
+
+		cluev->padding = style->padding;
+
+		cluev->border_style = style->border_style;
+		cluev->border_width = style->border_width;
+		cluev->border_color = style->border_color;
+		if (cluev->border_color)
+			html_color_ref (cluev->border_color);
+
+		cluev->background_color = style->bg_color;
+		if (cluev->background_color)
+			html_color_ref (cluev->background_color);
+	} else {
+		if (cluev->border_color)
+			html_color_unref (cluev->border_color);
+
+		if (cluev->border_color)
+			html_color_unref (cluev->background_color);
+
+		cluev->border_style = HTML_BORDER_NONE;
+		cluev->border_width = 0;
+		cluev->border_color = NULL;
+		cluev->background_color = NULL;
+	}
+}
