@@ -41,6 +41,7 @@
 #include "htmlobject.h"
 #include "htmlpainter.h"
 #include "htmltable.h"
+#include "htmltablecell.h"
 #include "htmltext.h"
 #include "htmlrule.h"
 #include "htmltype.h"
@@ -642,7 +643,7 @@ html_object_real_cursor_backward (HTMLObject *self, HTMLCursor *cursor)
 }
 
 static gboolean
-html_object_real_cursor_right (HTMLObject *self, HTMLCursor *cursor)
+html_object_real_cursor_right (HTMLObject *self, HTMLPainter *painter, HTMLCursor *cursor)
 {
 	HTMLDirection dir = html_object_get_direction (self);
 
@@ -652,7 +653,7 @@ html_object_real_cursor_right (HTMLObject *self, HTMLCursor *cursor)
 	if (html_object_is_container (self))
 		return FALSE;
 
-	if (dir == HTML_DIRECTION_LTR) {
+	if (dir != HTML_DIRECTION_RTL) {
 		gint len;
 
 		len = html_object_get_length (self);
@@ -677,7 +678,7 @@ html_object_real_cursor_right (HTMLObject *self, HTMLCursor *cursor)
 }
 
 static gboolean
-html_object_real_cursor_left (HTMLObject *self, HTMLCursor *cursor)
+html_object_real_cursor_left (HTMLObject *self, HTMLPainter *painter, HTMLCursor *cursor)
 {
 	HTMLDirection dir = html_object_get_direction (self);
 
@@ -687,7 +688,7 @@ html_object_real_cursor_left (HTMLObject *self, HTMLCursor *cursor)
 	if (html_object_is_container (self))
 		return FALSE;
 
-	if (dir == HTML_DIRECTION_LTR) {
+	if (dir != HTML_DIRECTION_RTL) {
 		HTMLObject *prev;
 		if (cursor->offset > 1 || (cursor->offset > 0 && (! (prev = html_object_prev_not_slave (self))
 								  || HTML_IS_CLUEALIGNED (prev) || !html_object_accepts_cursor (prev)))) {
@@ -711,13 +712,13 @@ html_object_real_cursor_left (HTMLObject *self, HTMLCursor *cursor)
 }
 
 static int
-html_object_real_get_right_edge_offset (HTMLObject *o, int offset)
+html_object_real_get_right_edge_offset (HTMLObject *o, HTMLPainter *painter, int offset)
 {
 	return html_object_get_length (o);
 }
 
 static int
-html_object_real_get_left_edge_offset (HTMLObject *o, int offset)
+html_object_real_get_left_edge_offset (HTMLObject *o, HTMLPainter *painter, int offset)
 {
 	return 0;
 }
@@ -1542,15 +1543,15 @@ html_object_cursor_backward (HTMLObject *self, HTMLCursor *cursor)
 }
 
 gboolean
-html_object_cursor_right (HTMLObject *self, HTMLCursor *cursor)
+html_object_cursor_right (HTMLObject *self, HTMLPainter *painter, HTMLCursor *cursor)
 {
-	return (* HO_CLASS (self)->cursor_right) (self, cursor);
+	return (* HO_CLASS (self)->cursor_right) (self, painter, cursor);
 }
 
 gboolean
-html_object_cursor_left (HTMLObject *self, HTMLCursor *cursor)
+html_object_cursor_left (HTMLObject *self, HTMLPainter *painter, HTMLCursor *cursor)
 {
-	return (* HO_CLASS (self)->cursor_left) (self, cursor);
+	return (* HO_CLASS (self)->cursor_left) (self, painter, cursor);
 }
 
 /*********************
@@ -2066,8 +2067,17 @@ html_object_get_insert_level (HTMLObject *o)
 {
 	switch (HTML_OBJECT_TYPE (o)) {
 	case HTML_TYPE_TABLECELL:
-	case HTML_TYPE_CLUEV:
-		return 3;
+	case HTML_TYPE_CLUEV: {
+		int level = 3;
+
+		while (o && (HTML_IS_CLUEV (o) || HTML_IS_TABLE_CELL (o))
+		       && HTML_CLUE (o)->head && (HTML_IS_CLUEV (HTML_CLUE (o)->head) || HTML_IS_TABLE_CELL (HTML_CLUE (o)->head))) {
+			level ++;
+			o = HTML_CLUE (o)->head;
+		}
+
+		return level;
+	}
 	case HTML_TYPE_CLUEFLOW:
 		return 2;
 	default:
@@ -2232,13 +2242,13 @@ html_object_get_flow (HTMLObject *o)
 }
 
 int
-html_object_get_right_edge_offset (HTMLObject *o, int offset)
+html_object_get_right_edge_offset (HTMLObject *o, HTMLPainter *painter, int offset)
 {
-	return (* HO_CLASS (o)->get_right_edge_offset) (o, offset);
+	return (* HO_CLASS (o)->get_right_edge_offset) (o, painter, offset);
 }
 
 int
-html_object_get_left_edge_offset (HTMLObject *o, int offset)
+html_object_get_left_edge_offset (HTMLObject *o, HTMLPainter *painter, int offset)
 {
-	return (* HO_CLASS (o)->get_left_edge_offset) (o, offset);
+	return (* HO_CLASS (o)->get_left_edge_offset) (o, painter, offset);
 }
