@@ -653,7 +653,7 @@ get_glyph_items_in_range (HTMLTextSlave *slave, HTMLPainter *painter, int start_
 GSList *
 html_text_slave_get_glyph_items (HTMLTextSlave *slave, HTMLPainter *painter)
 {
-	if (!slave->glyph_items || (HTML_OBJECT (slave)->change & HTML_CHANGE_RECALC_PI)) {
+	if (painter && (!slave->glyph_items || (HTML_OBJECT (slave)->change & HTML_CHANGE_RECALC_PI))) {
 		clear_glyph_items (slave);
 
 		HTML_OBJECT (slave)->change &= ~HTML_CHANGE_RECALC_PI;
@@ -828,7 +828,7 @@ draw_text (HTMLTextSlave *self,
 												     p, HTMLSpellErrorColor)->color);
 				        /* printf ("spell error: %s\n", html_text_get_text (slave->owner, off)); */
 			
-					html_painter_draw_spell_error (p, obj->x + tx + run_width + html_painter_pango_to_engine (p, start_x),
+					html_painter_draw_spell_error (p, obj->x + tx + html_painter_pango_to_engine (p, run_width + start_x),
 								       obj->y + ty + get_ys (self->owner, p), html_painter_pango_to_engine (p, width));
 				}
 			}
@@ -1167,7 +1167,7 @@ html_text_slave_is_index_in_glyph (HTMLTextSlave *slave, HTMLTextSlave *next_sla
 }
 
 static HTMLTextSlaveGlyphItem *
-html_text_slave_get_glyph_item_at_offset (HTMLTextSlave *slave, int offset, HTMLTextSlaveGlyphItem **prev, HTMLTextSlaveGlyphItem **next, int *start_width, int *index_out)
+html_text_slave_get_glyph_item_at_offset (HTMLTextSlave *slave, HTMLPainter *painter, int offset, HTMLTextSlaveGlyphItem **prev, HTMLTextSlaveGlyphItem **next, int *start_width, int *index_out)
 {
 	HTMLTextSlaveGlyphItem *rv = NULL;
 	HTMLTextSlaveGlyphItem *prev_gi, *next_gi;
@@ -1182,7 +1182,7 @@ html_text_slave_get_glyph_item_at_offset (HTMLTextSlave *slave, int offset, HTML
 	if (start_width)
 		*start_width = 0;
 
-	cur = html_text_slave_get_glyph_items (slave, NULL);
+	cur = html_text_slave_get_glyph_items (slave, painter);
 	if (cur) {
 		for (prev_gi = NULL; cur; cur = cur->next) {
 			HTMLTextSlaveGlyphItem *gi = (HTMLTextSlaveGlyphItem *) cur->data;
@@ -1255,11 +1255,11 @@ html_text_slave_gi_right_edge (HTMLTextSlave *slave, HTMLCursor *cursor, HTMLTex
 }
 
 static gboolean
-html_text_slave_cursor_right_one (HTMLTextSlave *slave, HTMLCursor *cursor)
+html_text_slave_cursor_right_one (HTMLTextSlave *slave, HTMLPainter *painter, HTMLCursor *cursor)
 {
 	HTMLTextSlaveGlyphItem *prev, *next;
 	int index;
-	HTMLTextSlaveGlyphItem *gi = html_text_slave_get_glyph_item_at_offset (slave, cursor->offset - slave->posStart, &prev, &next, NULL, &index);
+	HTMLTextSlaveGlyphItem *gi = html_text_slave_get_glyph_item_at_offset (slave, painter, cursor->offset - slave->posStart, &prev, &next, NULL, &index);
 
 	if (!gi)
 		return FALSE;
@@ -1274,7 +1274,7 @@ html_text_slave_cursor_right_one (HTMLTextSlave *slave, HTMLCursor *cursor)
 		}
 	} else {
 		/* RTL */
-		if (index > gi->glyph_item.item->offset && (index <= gi->glyph_item.item->offset + gi->glyph_item.item->length)) {
+		if (index > gi->glyph_item.item->offset && index <= gi->glyph_item.item->offset + gi->glyph_item.item->length) {
 			cursor->offset --;
 			cursor->position --;
 
@@ -1302,24 +1302,25 @@ html_text_slave_cursor_right_one (HTMLTextSlave *slave, HTMLCursor *cursor)
 }
 
 gboolean
-html_text_slave_cursor_right (HTMLTextSlave *slave, HTMLCursor *cursor)
+html_text_slave_cursor_right (HTMLTextSlave *slave, HTMLPainter *painter, HTMLCursor *cursor)
 {
-	HTMLTextPangoInfo *pi = html_text_get_pango_info (slave->owner, NULL);
+	HTMLTextPangoInfo *pi = html_text_get_pango_info (slave->owner, painter);
 	gboolean step_success;
 
 	do
-		step_success = html_text_slave_cursor_right_one (slave, cursor);
+		step_success = html_text_slave_cursor_right_one (slave, painter, cursor);
 	while (step_success && !pi->attrs [cursor->offset].is_cursor_position);
 
 	return step_success;
 }
 
 static gboolean
-html_text_slave_cursor_left_one (HTMLTextSlave *slave, HTMLCursor *cursor)
+html_text_slave_cursor_left_one (HTMLTextSlave *slave, HTMLPainter *painter, HTMLCursor *cursor)
 {
 	HTMLTextSlaveGlyphItem *prev, *next;
 	int index;
-	HTMLTextSlaveGlyphItem *gi = html_text_slave_get_glyph_item_at_offset (slave, cursor->offset - slave->posStart, &prev, &next, NULL, &index);
+	HTMLObject *prev_obj = HTML_OBJECT (slave->owner)->prev;
+	HTMLTextSlaveGlyphItem *gi = html_text_slave_get_glyph_item_at_offset (slave, painter, cursor->offset - slave->posStart, &prev, &next, NULL, &index);
 
 /* 	printf ("gi: %p item num chars: %d\n", gi, gi ? gi->glyph_item.item->num_chars : -1); */
 
@@ -1328,7 +1329,7 @@ html_text_slave_cursor_left_one (HTMLTextSlave *slave, HTMLCursor *cursor)
 
 	if (gi->glyph_item.item->analysis.level % 2 == 0) {
 		/* LTR */
-		if (index - gi->glyph_item.item->offset > 1 || (!prev && index - gi->glyph_item.item->offset > 0)) {
+		if (index - gi->glyph_item.item->offset > 1 || (!prev && !prev_obj && index - gi->glyph_item.item->offset > 0)) {
 			cursor->offset --;
 			cursor->position --;
 
@@ -1366,56 +1367,72 @@ html_text_slave_cursor_left_one (HTMLTextSlave *slave, HTMLCursor *cursor)
 }
 
 gboolean
-html_text_slave_cursor_left (HTMLTextSlave *slave, HTMLCursor *cursor)
+html_text_slave_cursor_left (HTMLTextSlave *slave, HTMLPainter *painter, HTMLCursor *cursor)
 {
-	HTMLTextPangoInfo *pi = html_text_get_pango_info (slave->owner, NULL);
+	HTMLTextPangoInfo *pi = html_text_get_pango_info (slave->owner, painter);
 	gboolean step_success;
 
 	do
-		step_success = html_text_slave_cursor_left_one (slave, cursor);
+		step_success = html_text_slave_cursor_left_one (slave, painter, cursor);
 	while (step_success && !pi->attrs [cursor->offset].is_cursor_position);
 
 	return step_success;
 }
 
 static gboolean
-html_text_slave_get_left_edge (HTMLTextSlave *slave, HTMLCursor *cursor)
+html_text_slave_get_left_edge (HTMLTextSlave *slave, HTMLPainter *painter, HTMLCursor *cursor)
 {
-	HTMLTextPangoInfo *pi = html_text_get_pango_info (slave->owner, NULL);
+	HTMLTextPangoInfo *pi = html_text_get_pango_info (slave->owner, painter);
+	int old_offset = cursor->offset;
+	int old_position = cursor->position;
 
-	cursor->offset = html_text_slave_get_left_edge_offset (slave);
+	cursor->offset = html_text_slave_get_left_edge_offset (slave, painter);
 
-	if (pi->attrs [cursor->offset].is_cursor_position)
+	if (pi->attrs [cursor->offset].is_cursor_position && old_offset != cursor->offset)
 		return TRUE;
-	else
-		return html_text_slave_cursor_right (slave, cursor);
+	else {
+		if (html_text_slave_cursor_right (slave, painter, cursor)) {
+			/* we should preserve position here as caller function correct position themselves */
+			cursor->position = old_position;
+			return TRUE;
+		} else
+			return FALSE;
+	}
 }
 
 static gboolean
-html_text_slave_get_right_edge (HTMLTextSlave *slave, HTMLCursor *cursor)
+html_text_slave_get_right_edge (HTMLTextSlave *slave, HTMLPainter *painter, HTMLCursor *cursor)
 {
-	HTMLTextPangoInfo *pi = html_text_get_pango_info (slave->owner, NULL);
+	HTMLTextPangoInfo *pi = html_text_get_pango_info (slave->owner, painter);
+	int old_offset = cursor->offset;
+	int old_position = cursor->position;
 
-	cursor->offset = html_text_slave_get_right_edge_offset (slave);
+	cursor->offset = html_text_slave_get_right_edge_offset (slave, painter);
 
-	if (pi->attrs [cursor->offset].is_cursor_position)
+	if (pi->attrs [cursor->offset].is_cursor_position && old_offset != cursor->offset)
 		return TRUE;
-	else
-		return html_text_slave_cursor_left (slave, cursor);
+	else {
+		if (html_text_slave_cursor_left (slave, painter, cursor)) {
+			/* we should preserve position here as caller function correct position themselves */
+			cursor->position = old_position;
+			return TRUE;
+		} else
+			return FALSE;
+	}
 }
 
 gboolean
-html_text_slave_cursor_head (HTMLTextSlave *slave, HTMLCursor *cursor)
+html_text_slave_cursor_head (HTMLTextSlave *slave, HTMLCursor *cursor, HTMLPainter *painter)
 {
-	if (html_text_slave_get_glyph_items (slave, NULL)) {
+	if (html_text_slave_get_glyph_items (slave, painter)) {
 		cursor->object = HTML_OBJECT (slave->owner);
 
 		if (html_text_get_pango_direction (slave->owner) != PANGO_DIRECTION_RTL) {
 			/* LTR */
-			return html_text_slave_get_left_edge (slave, cursor);
+			return html_text_slave_get_left_edge (slave, painter, cursor);
 		} else {
 			/* RTL */
-			return html_text_slave_get_right_edge (slave, cursor);
+			return html_text_slave_get_right_edge (slave, painter, cursor);
 		}
 	}
 
@@ -1423,17 +1440,17 @@ html_text_slave_cursor_head (HTMLTextSlave *slave, HTMLCursor *cursor)
 }
 
 gboolean
-html_text_slave_cursor_tail (HTMLTextSlave *slave, HTMLCursor *cursor)
+html_text_slave_cursor_tail (HTMLTextSlave *slave, HTMLCursor *cursor, HTMLPainter *painter)
 {
-	if (html_text_slave_get_glyph_items (slave, NULL)) {
+	if (html_text_slave_get_glyph_items (slave, painter)) {
 		cursor->object = HTML_OBJECT (slave->owner);
 
 		if (html_text_get_pango_direction (slave->owner) != PANGO_DIRECTION_RTL) {
 			/* LTR */
-			return html_text_slave_get_right_edge (slave, cursor);
+			return html_text_slave_get_right_edge (slave, painter, cursor);
 		} else {
 			/* RTL */
-			return html_text_slave_get_left_edge (slave, cursor);
+			return html_text_slave_get_left_edge (slave, painter, cursor);
 		}
 	}
 
@@ -1448,7 +1465,7 @@ html_text_slave_get_cursor_base (HTMLTextSlave *slave, HTMLPainter *painter, gui
 
 	html_object_calc_abs_position (HTML_OBJECT (slave), x, y);
 
-	gi = html_text_slave_get_glyph_item_at_offset (slave, (int) offset, NULL, NULL, &start_width, &index);
+	gi = html_text_slave_get_glyph_item_at_offset (slave, painter, (int) offset, NULL, NULL, &start_width, &index);
 
 /* 	printf ("gi: %p index: %d start_width: %d item indexes %d %d\n", */
 /* 		gi, index, start_width, gi ? gi->glyph_item.item->offset : -1, */
@@ -1465,9 +1482,9 @@ html_text_slave_get_cursor_base (HTMLTextSlave *slave, HTMLPainter *painter, gui
 }
 
 int
-html_text_slave_get_left_edge_offset (HTMLTextSlave *slave)
+html_text_slave_get_left_edge_offset (HTMLTextSlave *slave, HTMLPainter *painter)
 {
-	GSList *gis = html_text_slave_get_glyph_items (slave, NULL);
+	GSList *gis = html_text_slave_get_glyph_items (slave, painter);
 
 	if (gis) {
 		HTMLTextSlaveGlyphItem *gi = (HTMLTextSlaveGlyphItem *) gis->data;
@@ -1477,8 +1494,9 @@ html_text_slave_get_left_edge_offset (HTMLTextSlave *slave)
 			return slave->posStart + g_utf8_pointer_to_offset (html_text_slave_get_text (slave), slave->owner->text + gi->glyph_item.item->offset);
 		} else {
 			/* RTL */
-			return slave->posStart + g_utf8_pointer_to_offset (html_text_slave_get_text (slave),
-									   slave->owner->text + gi->glyph_item.item->offset + gi->glyph_item.item->length);
+			return slave->posStart + MIN (slave->posLen, g_utf8_pointer_to_offset (html_text_slave_get_text (slave),
+											       slave->owner->text
+											       + gi->glyph_item.item->offset + gi->glyph_item.item->length));
 		}
 	} else {
 		if (slave->owner->text_len > 0)
@@ -1489,17 +1507,18 @@ html_text_slave_get_left_edge_offset (HTMLTextSlave *slave)
 }
 
 int
-html_text_slave_get_right_edge_offset (HTMLTextSlave *slave)
+html_text_slave_get_right_edge_offset (HTMLTextSlave *slave, HTMLPainter *painter)
 {
-	GSList *gis = html_text_slave_get_glyph_items (slave, NULL);
+	GSList *gis = html_text_slave_get_glyph_items (slave, painter);
 
 	if (gis) {
 		HTMLTextSlaveGlyphItem *gi = (HTMLTextSlaveGlyphItem *) g_slist_last (gis)->data;
 
 		if (gi->glyph_item.item->analysis.level % 2 == 0) {
 			/* LTR */
-			return slave->posStart + g_utf8_pointer_to_offset (html_text_slave_get_text (slave),
-									   slave->owner->text + gi->glyph_item.item->offset + gi->glyph_item.item->length);
+			return slave->posStart + MIN (slave->posLen, g_utf8_pointer_to_offset (html_text_slave_get_text (slave),
+											       slave->owner->text
+											       + gi->glyph_item.item->offset + gi->glyph_item.item->length));
 		} else {
 			/* RTL */
 			return slave->posStart + g_utf8_pointer_to_offset (html_text_slave_get_text (slave), slave->owner->text + gi->glyph_item.item->offset);
